import datetime
from typing import Any


class Event:
    def __init__(self, calendar_data: dict[str, Any], event_data: dict[str, Any]):
        self.color = self._hex_to_color(calendar_data["backgroundColor"])

        start = event_data["start"]
        self.summary = event_data["summary"]
        if "dateTime" in start:
            self.start_time = datetime.datetime.strptime(
                start["dateTime"], "%Y-%m-%dT%H:%M:%S%z",
            )
            self.all_day = False
        else:
            self.start_time = datetime.datetime.strptime(start["date"], "%Y-%m-%d")
            self.all_day = True

    def __eq__(self, other: object) -> bool:
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    def _hex_to_color(self, hex_: str) -> tuple[int, int, int]:
        hex_ = hex_.replace("#", "")
        return int(hex_[:2], 16), int(hex_[2:4], 16), int(hex_[4:6], 16)



from abc import ABC, abstractmethod
from queue import Empty, Queue
from threading import Thread

from inky import Inky7Colour
from PIL import Image


class Display(ABC):
    @abstractmethod
    def set_image(self, image: Image.Image) -> None:
        ...

    @abstractmethod
    def check_health(self) -> None:
        ...

    @abstractmethod
    def close(self) -> None:
        ...


class MockDisplay(Display):
    def set_image(self, image: Image.Image) -> None:
        image.show("eInk Calendar")

    def check_health(self) -> None:
        pass

    def close(self) -> None:
        pass


class EInkDisplay(Display):
    def __init__(self) -> None:
        self._inky = Inky7Colour()
        self._image_queue = Queue[Image.Image]()
        self._running = True
        self._thread = Thread(target=self._run, daemon=True)
        self._thread.start()

    def set_image(self, image: Image.Image) -> None:
        self._image_queue.put(image)

    def check_health(self) -> None:
        if not self._thread.is_alive():
            raise RuntimeError("EInkDisplay thread died")

    def close(self) -> None:
        self._running = False
        self._thread.join()

    def _run(self) -> None:
        while self._running:
            try:
                image = self._image_queue.get(timeout=1)
                self._inky.set_image(image)
                self._inky.show()
            except Empty:
                pass



from PIL import ImageDraw, ImageFont

from eink_calendar import api, ui


class Event:
    HEIGHT = 40
    PADDING_BOTTOM = 10
    START_X = 65
    START_Y = 65
    COLOR_SQUARE_SIZE = 15
    TEXT_X_OFFSET = 30
    TIME_Y_OFFSET = 18

    def __init__(
        self,
        draw: ImageDraw.ImageDraw,
        font: ImageFont.FreeTypeFont,
        api_event: api.Event,
        y: int,
    ):
        self._draw = draw
        self.x = self.START_X
        self.y = y
        self.color = api_event.color

        if api_event.all_day:
            time_str = "All Day"
        else:
            time_str = api_event.start_time.strftime("%I:%M %p")

        self.time_text = ui.Text(
            draw=draw,
            font=font,
            text=time_str,
            color=(0, 0, 0),
            x=self.x + self.TEXT_X_OFFSET,
            y=self.y,
            max_chars=60,
        )
        self.ui_text = ui.Text(
            draw=draw,
            font=font,
            text=api_event.summary,
            color=(0, 0, 0),
            x=self.x + self.TEXT_X_OFFSET,
            y=self.y + self.TIME_Y_OFFSET,
            max_chars=60,
        )

    def draw(self) -> None:
        rect_x = self.x
        rect_y = int(self.y + (self.HEIGHT / 2 - self.COLOR_SQUARE_SIZE / 2))
        self._draw.rectangle(
            (
                rect_x,
                rect_y,
                rect_x + self.COLOR_SQUARE_SIZE,
                rect_y + self.COLOR_SQUARE_SIZE,
            ),
            fill=self.color,
        )

        self.time_text.draw()
        self.ui_text.draw()



import datetime
import logging
import sys
from pathlib import Path

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from xdg import BaseDirectory

from .event import Event

SCOPES = ["https://www.googleapis.com/auth/calendar.readonly"]


class Client:
    def __init__(self) -> None:
        self._creds = self._get_credentials()
        self._service = build("calendar", "v3", credentials=self._creds)

    def get_events(self) -> list[Event]:
        calendars = self._service.calendarList().list().execute()

        now = datetime.datetime.utcnow()
        time_min = now - datetime.timedelta(hours=20)

        # Only look at calendars the user selects on their Google Calendar view
        selected_calendars = [c for c in calendars["items"] if c.get("selected", False)]

        events = []
        for calendar in selected_calendars:
            result = (
                self._service.events()
                .list(
                    calendarId=calendar["id"],
                    timeMin=time_min.isoformat() + "Z",
                    maxResults=10,
                    singleEvents=True,
                    orderBy="startTime",
                )
                .execute()
            )
            events += [Event(calendar, e) for e in result.get("items", [])]

        # Filter for today's events
        events = [e for e in events if e.start_time.date() == datetime.date.today()]

        return self._sort_events(events)

    def _get_credentials(self) -> Credentials:
        creds = None
        data_path = Path(BaseDirectory.save_data_path("eink-calendar"))

        credentials_file = data_path / "credentials.json"
        if not credentials_file.is_file():
            logging.error(f"Missing credentials file at: {credentials_file}")
            sys.exit(1)

        token = data_path / "token.json"
        if token.is_file():
            creds = Credentials.from_authorized_user_file(str(token), SCOPES)
        if creds is None or not creds.valid:
            if creds is not None and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    str(credentials_file), SCOPES,
                )
                creds = flow.run_local_server(port=0)
            token.write_text(creds.to_json())

        return creds

    def _sort_events(self, events: list[Event]) -> list[Event]:
        """Sorts events by their start time, with all-day events always at the start."""
        all_days = [e for e in events if e.all_day]
        all_days = sorted(all_days, key=lambda e: e.start_time)
        non_all_days = [e for e in events if not e.all_day]
        non_all_days = sorted(non_all_days, key=lambda e: e.start_time)

        return all_days + non_all_days



import datetime
import faulthandler
import logging
import signal
from argparse import ArgumentParser
from pathlib import Path
from types import FrameType

from PIL import Image, ImageDraw, ImageFont

from . import api, ui
from .display import EInkDisplay, MockDisplay

SCREEN_WIDTH = 600
SCREEN_HEIGHT = 448


def main() -> int:
    faulthandler.enable()

    logging.basicConfig(level=logging.INFO)

    parser = ArgumentParser(
        description="Displays calendar events on an Inky 7-color display",
    )

    parser.add_argument(
        "--no-display",
        action="store_true",
        help="Do not communicate with an eInk display",
    )

    args = parser.parse_args()

    api_client = api.Client()

    font_file = Path(__file__).parent / "lora.ttf"
    day_font = ImageFont.truetype(str(font_file), 36)
    times_font = ImageFont.truetype(str(font_file), 16)

    stop = False

    def on_sigint(_sig: int, _frame: FrameType | None) -> None:
        nonlocal stop
        stop = True
    signal.signal(signal.SIGINT, on_sigint)

    event_stream = api.EventStream(api_client)

    ui_events: list[ui.Event] = []

    display = MockDisplay() if args.no_display else EInkDisplay()

    while not stop:
        event_stream.check_health()
        display.check_health()

        image = Image.new("RGB", (SCREEN_WIDTH, SCREEN_HEIGHT), (255, 255, 255))
        draw = ImageDraw.Draw(image)

        today = datetime.date.today()
        day_text = ui.Text(
            draw=draw,
            font=day_font,
            text=today.strftime("%A, %B %d"),
            color=(0, 0, 0),
        )
        day_text.set_position(int(SCREEN_WIDTH / 2 - day_text.width / 2), 1)
        day_text.draw()

        for ui_event in ui_events:
            ui_event.draw()

        api_events = event_stream.get()
        if api_events is not None:
            ui_events = []

            for api_event in api_events:
                ui_event = ui.Event(
                    draw,
                    times_font,
                    api_event,
                    (ui.Event.HEIGHT + ui.Event.PADDING_BOTTOM) * len(ui_events)
                    + ui.Event.START_Y,
                )
                ui_event.draw()
                ui_events.append(ui_event)

            display.set_image(image)

    logging.info("Waiting for event stream to stop...")
    event_stream.close()
    logging.info("Waiting for display to finish up...")
    display.close()

    return 0



import logging
from queue import Empty, Queue
from threading import Thread
from time import sleep, time
from typing import Optional

from httplib2 import HttpLib2Error

from .client import Client
from .event import Event


class EventStream:
    def __init__(self, client: Client, poll_delay: float = 60):
        self._client = client
        self._poll_delay = poll_delay
        self._output = Queue[list[Event]]()
        self._running = True
        self._thread = Thread(target=self._run, daemon=True)
        self._thread.start()

    def get(self) -> Optional[list[Event]]:
        try:
            return self._output.get_nowait()
        except Empty:
            return None

    def check_health(self) -> None:
        if not self._thread.is_alive():
            raise RuntimeError("EventStream thread died")

    def close(self) -> None:
        self._running = False
        self._thread.join()

    def _run(self) -> None:
        last_poll = 0.0
        last_api_events = None

        while self._running:
            sleep(0.01)

            if time() - last_poll < self._poll_delay:
                continue

            logging.info("Polling for events")
            try:
                api_events = self._client.get_events()
                if api_events != last_api_events:
                    logging.info("Event changes detected")
                    last_api_events = api_events
                    self._output.put(api_events)
            except HttpLib2Error as ex:
                logging.error(f"Could not get events: {ex}")

            last_poll = time()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eink_calendar', 'eink_calendar.api', 'eink_calendar.ui']

package_data = \
{'': ['*']}

install_requires = \
['google-api-python-client>=2.100.0,<3.0.0',
 'google-auth-httplib2>=0.1.1,<0.2.0',
 'google-auth-oauthlib>=1.1.0,<2.0.0',
 'inky[example-depends,rpi]>=1.5.0,<2.0.0',
 'pillow>=10.0.1,<11.0.0',
 'pyxdg>=0.28,<0.29']

entry_points = \
{'console_scripts': ['eink-calendar = eink_calendar.__main__:main']}

setup_kwargs = {
    'name': 'eink-calendar',
    'version': '0.2.2',
    'description': 'Shows Google Calendar events on an Inky Impression display',
    'long_description': "# eInk Calendar\n\nDisplays Google Calendar events on an Inky Impression display connected to a\nRaspberry Pi.\n\n![Example showing how the UI looks](https://i.imgur.com/cKqnSmU.png)\n\n## Installation\n\nThis project is available on PyPI and can be installed with `pipx`.\n\n```bash\npipx install eink-calendar\n```\n\nThen, you will need to create a desktop OAuth2 client ID using the Google Cloud\nconsole. For details on how to do this, see\n[Google's Documentation][oauth2_client_docs].\n\nOnce you have created the client ID, download the client secret JSON file to\n`~/.local/share/eink-calendar/credentials.json`.\n\nThen, start the application:\n\n```bash\neink-calendar\n```\n\nA browser window will automatically open prompting you to give eInk Calendar\nread access to your calendars. Once this process completes, the application\nwill start. You only need to complete this process once.\n\n## Development\n\nIf you want to develop without the eInk display connected, start the\napplication with the following flag:\n\n```bash\neink-calendar --no-display\n```\n\nThe generated images will be opened using your image viewer instead.\n\n[oauth2_client_docs]: https://developers.google.com/identity/protocols/oauth2/native-app\n",
    'author': 'Tyler Compton',
    'author_email': 'xaviosx@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from PIL import ImageDraw, ImageFont


class Text:
    def __init__(
        self,
        *,
        draw: ImageDraw.ImageDraw,
        font: ImageFont.FreeTypeFont,
        text: str,
        color: tuple[int, int, int],
        x: int = 0,
        y: int = 0,
        max_chars: int = 0,
    ):
        if max_chars > 0 and len(text) > max_chars:
            text = text[: max_chars - 3] + "..."

        self._draw = draw
        self._font = font
        self._text = text
        self._color = color
        self._x = x
        self._y = y

        self.width = draw.textlength(text, font=font)

    def draw(self) -> None:
        self._draw.text(
            (self._x, self._y), self._text, fill=self._color, font=self._font,
        )

    def set_position(self, x: int, y: int) -> None:
        self._x = x
        self._y = y
